from .automl import init as automlapi_init
automlapi_init()
from .automl_batch import *
from .automl_cloudwatch import *
from .automl_cognito import *
from .automl_rds import *
from .automl_codebuild import *
from .automl_ec2 import *
from .automl_elb import *
from .automl_ecs import *
from .automl_s3 import *
from .automlClient import APIUser, private_upload_dataset, make_lst



import boto3
from .automl import AWS_ACC_KEY_ID, AWS_SEC_ACC_KEY, AWS_REGION_NAME

client_elb = boto3.client('elbv2',
						aws_access_key_id=AWS_ACC_KEY_ID,
						aws_secret_access_key=AWS_SEC_ACC_KEY,
						region_name=AWS_REGION_NAME)

def create_flask_target_group(name):
	response = client_elb.create_target_group(
	    Name=name,
	    Protocol='HTTP',
	    Port=5000,
	    VpcId='vpc-9c6422fa',
	    HealthCheckProtocol='HTTP',
	    HealthCheckPort='traffic-port',
	    HealthCheckEnabled=True,
	    HealthCheckPath='/healthcheck',
	    HealthCheckIntervalSeconds=30,
	    HealthCheckTimeoutSeconds=5,
	    HealthyThresholdCount=3,
	    UnhealthyThresholdCount=3,
	    Matcher={
	        'HttpCode': '200'
	    },
	    TargetType='instance',

	)
	if int(response['ResponseMetadata']['HTTPStatusCode']) == 200:
		print(f'create_flask_target_group : INFO : Created TargetGroup {name}!')
		return str(response['TargetGroups'][0]['TargetGroupArn'])
	else:
		print(f'create_flask_target_group : ERROR : Could not create TargetGroup {name}!')
		return False

def add_listener_to_load_balancer(port, mappings):
	response = client_elb.create_listener(
	    LoadBalancerArn='arn:aws:elasticloadbalancing:us-west-2:749868801319:loadbalancer/app/Flask-Services-LB/fc15f78a9d19b599',
	    Protocol='HTTP',
	    Port=port,
	    DefaultActions=[
	        {
	            'Type': 'fixed-response',
	            'FixedResponseConfig': {
	                'MessageBody': f'Usage: invoke a service from {list(mappings.keys())}',
	                'StatusCode': '503',
	                'ContentType': 'text/plain'
	            }
	        }
	    ]
	)

	if int(response['ResponseMetadata']['HTTPStatusCode']) == 200:
		print(f'add_listener_to_load_balancer : INFO : Added listener on port {port}!')
		listener_arn = response['Listeners'][0]['ListenerArn']
		print(f'add_listener_to_load_balancer : INFO : Adding traffic redirection rules...')
		return add_rules(listener_arn, mappings)
	return False

def add_rules(listener_arn, mappings):
	p = 0
	for path in mappings:
		p = p + 1
		response = client_elb.create_rule(
		    ListenerArn=listener_arn,
		    Conditions=[
		        {
		            'Field': 'path-pattern',
		            'PathPatternConfig': {
		                'Values': [f'/{path}']
		            }
		        }
		    ],
			Priority=p,
		    Actions=[
		        {
		            'Type': 'forward',
		            'ForwardConfig': {
		                'TargetGroups': [
		                    {
		                        'TargetGroupArn': mappings[path],
		                        'Weight': 1
		                    }
		                ]
		            }
		        }
		    ]
		)
		if response['ResponseMetadata']['HTTPStatusCode'] != 200:
			print(f'add_rules : ERROR : Oops! Could not add redirection rule for {path} -> {mappings[path]}')
			return False
		else:
			print(f'add_rules : INFO : Added rule for {path} -> {mappings[path]}')
	return True

def get_targets_health(target_group_arn):
	targets_health = client.describe_target_health(TargetGroupArn=target_group_arn)['TargetHealthDescriptions']
	return [x['TargetHealth']['State'] for x in targets_health]

def get_num_unhealthy_targets(target_group_arn):
	targets_health = get_targets_health(target_group_arn)
	return len(list(filter(lambda x: x.lower() == 'unhealthy', targets_health)))



# import MySQLdb as mysql
import pymysql as mysql
import os
import hashlib
import json
from .automl import BD_HOST, BD_PASS, BD_DATABASE, BD_USER

def hash_password(password):
    n = 100000
    hashed_password = hashlib.pbkdf2_hmac('sha256', password.encode(), b'salt', n).hex()
    return hashed_password

def run_exists(query):

    exists = False

    try:
        db = mysql.connect(host=BD_HOST,
                database=BD_DATABASE,
                user=BD_USER,
                password=BD_PASS)
        cursor = db.cursor()
        cursor.execute(query)
        response = cursor.fetchall()
        exists = len(response) > 0
    except Exception as e:
        print(f"run_exists - ERROR - {e}")
    finally:
        db.close()

    return exists

def run_select(query):
    result = []
    try:
        db = mysql.connect(host=BD_HOST,
                database=BD_DATABASE,
                user=BD_USER,
                password=BD_PASS)
        cursor = db.cursor()
        cursor.execute(query)
        response = cursor.fetchall()
        field_names = [i[0] for i in cursor.description]
        for elem in response:
            result.append({})
            for index, value in enumerate(elem):
                result[-1][field_names[index]] = value
    except Exception as e:
        print(f"run_select - ERROR - {e}")
    finally:
        db.close()
    return result

def run_insert(query):
	pk = -1
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		cursor = db.cursor()
		cursor.execute(query)
		pk = cursor.lastrowid
		db.commit()
	except mysql.IntegrityError:
		pass
	except Exception as e:
		print(f"run_insert : ERROR :  {e}")
	finally:
		db.close()
	return pk

def run_update(query):
    try:
        db = mysql.connect(host=BD_HOST,
                database=BD_DATABASE,
                user=BD_USER,
                password=BD_PASS)

        cursor = db.cursor()
        if isinstance(query, list):
            for subquery in query:
                cursor.execute(subquery)
        else:
            cursor.execute(query)
        db.commit()
    except Exception as e:
        print("run_update - ERROR " + str(e))
    finally:
        db.close()

def run_delete(query):
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)

		cursor = db.cursor()
		cursor.execute(query)
		db.commit()
	except Exception as e:
		print("run_delete - ERROR " + str(e))
	finally:
		db.close()

def run_multiple_insert(queries):
    pks = []
    try:
        db = mysql.connect(host=BD_HOST,
                            database=BD_DATABASE,
                            user=BD_USER,
                            password=BD_PASS)
        cursor = db.cursor()
        for query in queries:
            cursor.execute(query)
            pks.append(cursor.lastrowid)
        db.commit()
    except Exception as e:
        print(f"run_multiple_insert : ERROR :  {e}")
    finally:
        db.close()
    return pks

def insert_n_objects(objectName, objects):

    table_name = f'neuralplatform_{objectName.lower()}'
    queries = []
    for object in objects:
        fields = ""
        values = ""
        for key in object:
            fields += key + ', '
            value = object[key]
            if isinstance(value, str):
                value = '"' + value + '"'
            values += str(value) + ', '
        else:
            fields = fields[:-2]
            values = values[:-2]
        queries.append(f'INSERT INTO {table_name}({fields}) VALUES({values});')

    return run_multiple_insert(queries)

def get_n_objects_by_key(objectName, n=1, key='id', keyValue=1):

    operator = '='
    if isinstance(keyValue, list):
        operator = 'IN'
        keyValue = '(' + str(keyValue)[1:-1] + ')'
    elif isinstance(keyValue, str):
        keyValue = '"' + keyValue + '"'
    query = f'SELECT * FROM neuralplatform_{objectName.lower()} WHERE {key} {operator} {keyValue};'
    elements = run_select(query)
    if len(elements) > 0:
        return elements[:n]
    return None

def update_object_by_conditions(objectName, fields, conditions):

    parsed_fields = ""
    parsed_conditions = ""
    for key in fields:
        value = fields[key]
        if isinstance(value, str):
            value = '"' + value + '"'
        parsed_fields += f'{key} = {value}, '
    else:
        parsed_fields = parsed_fields[:-2]

    for key in conditions:
        operator = '='
        value = conditions[key]
        if isinstance(value, list):
            operator = 'IN'
            value = '(' + str(value)[1:-1] + ')'
        elif isinstance(value, str):
            value = '"' + value + '"'
        parsed_conditions += f'{key} {operator} {value} AND '
    else:
        parsed_conditions = parsed_conditions[:-4]
    query = f'UPDATE neuralplatform_{objectName.lower()} SET {parsed_fields} WHERE {parsed_conditions};'
    run_update(query)

def get_pagesInfo_of_request(request_id):
    pagesInfo = {"pagesInfo": []}
    request   = get_n_objects_by_key('request', 1, 'id', request_id)[0]
    pages     = get_n_objects_by_key('page', None, 'document_id', int(request['document_id'])) or []
    for page in pages:
        class_name = ""
        if page['tagged']:
            classification = get_n_objects_by_key('classification', 1, 'page_id', int(page['id']))[0]
            class_id = int(classification['classDefinition_id'])
            class_name = get_n_objects_by_key('classDefinition', 1, 'id', class_id)[0]['name']
        pagesInfo['pagesInfo'].append({"uri": page['imgUri'], "class": class_name})
    return pagesInfo

def update_object_by_key(objectName, key='id', keyValue=1, fields=None):

    if fields:
        operator = '='
        if isinstance(keyValue, list):
            operator = 'IN'
            keyValue = '(' + str(keyValue)[1:-1] + ')'
        elif isinstance(keyValue, str):
            keyValue = '"' + keyValue + '"'
        changes = ""
        for field in fields:
            value = fields[field]
            if isinstance(value, str):
                value = "'" + value + "'"
            changes += field + ' = ' + str(value) + ', '
        else:
            changes = changes[:-2]
        query = f'UPDATE neuralplatform_{objectName.lower()} SET {changes} WHERE {key} {operator} {keyValue};'
        run_update(query)

def get_projects_of_projectManager(projectManager_id):
	query = f"SELECT project_id FROM neuralplatform_projectmanagerassignedproject WHERE projectManager_id = {projectManager_id};"
	project_ids = run_select(query)
	return project_ids

def get_bucketName_by_page_id(page_id):
    try:
        query = "SELECT s3Bucket FROM neuralplatform_account WHERE id = " + \
            "(SELECT account_id FROM neuralplatform_project WHERE id = " + \
            "(SELECT project_id FROM neuralplatform_document WHERE id = " + \
            f"(SELECT document_id FROM neuralplatform_page WHERE id = {page_id})));"
        return run_select(query)[0]['s3Bucket']
    except Exception as e:
        print(f"get_bucketName_by_page_id : Error : {e}")
        return ""

def validate_projectManager(account_code, username, hashed_password):
    accounts = get_n_objects_by_key('account', 1, 'code', account_code)
    if accounts:
        account_id = int(accounts[0]['id'])
        projectManagers = get_n_objects_by_key('projectmanager', None, 'account_id', account_id)
        if projectManagers:
            for pm in projectManagers:
                if pm['username'] == username and pm['password'] == hashed_password:
                    return True
    return False

def validate_admin(account_code, username, hashed_password):
    accounts = get_n_objects_by_key('account', 1, 'code', account_code)
    if accounts:
        account = accounts[0]
        return account['username'] == username and account['password'] == hashed_password
    return False

def validate_user(account_code, username, password):
    hashed_password = hash_password(password)
    return validate_admin(account_code, username, hashed_password) or validate_projectManager(account_code, username, hashed_password)

def insert_document(uploadDate, filename, extension, phase, uri, nPages, training, dataset_id, uploadMethod_id, project_id):
	query = f'INSERT INTO neuralplatform_document(uploadDate, name, extension, phase, uri, nPages, training, dataset_id, uploadMethod_id, project_id) ' + \
			f'VALUES ("{uploadDate}", "{filename}", "{extension}", "{phase}", "{uri}", {nPages}, {training}, {dataset_id}, {uploadMethod_id}, {project_id});'
	return run_insert(query)

def insert_request(phase, requestDate, document_id, project_id):
    emptyJson = '"{}"'
    query = f'INSERT INTO neuralplatform_request(phase, requestDate, document_id, project_id, response, status) ' + \
            f'VALUES ("{phase}", "{requestDate}", {document_id}, {project_id}, {emptyJson}, "pending");'
    return run_insert(query)

def insert_page(imgUri, ocrUri, document_id):
    query = f'INSERT INTO neuralplatform_page(imgUri, ocrUri, document_id) ' + \
            f'VALUES ("{imgUri}", "{ocrUri}", {document_id});'
    return run_insert(query)

def get_pending_and_unblocked_steps():
    # TODO: # OPTIMIZE:  this
    # query1 = 'SELECT neuralplatform_step.id, neuralplatform_request.requestDate FROM neuralplatform_step INNER JOIN ' + \
    #          'neuralplatform_request ON neuralplatform_step.request_id=neuralplatform_request.id WHERE neuralplatform_step.status = "pending" ORDER BY neuralplatform_request.requestDate ASC;'
    unblocked_steps = []
    querySD = 'SELECT id, blockingStep_id FROM neuralplatform_stepdefinition;'
    stepDefinitions = run_select(querySD)
    dependencies = {s['id']: s['blockingStep_id'] for s in stepDefinitions}
    for request in run_select('SELECT * FROM neuralplatform_request WHERE status = "running" ORDER BY id ASC;'):
        query1 = f'SELECT * FROM neuralplatform_step WHERE status = "pending" AND request_id = {int(request["id"])};'
        pending_steps = run_select(query1)
        if len(pending_steps) > 0:
            pending_stepDefinitions = list(set([x['stepDefinition_id'] for x in pending_steps]))
            unblocked_stepDefinitions = []
            for pending_stepDefinition in pending_stepDefinitions:
                sd_dependency = dependencies[pending_stepDefinition] or -1
                if not run_exists(f'SELECT id FROM neuralplatform_step WHERE status NOT IN ("succeeded") AND stepDefinition_id = {sd_dependency} AND request_id = {int(request["id"])};'):
                    unblocked_stepDefinitions.append(pending_stepDefinition)
            unblocked_steps += list(filter(lambda x: x['stepDefinition_id'] in unblocked_stepDefinitions, pending_steps))
    return unblocked_steps

def classify_page(page_id, class_id):
    if run_exists(f"SELECT DISTINCT 1 FROM neuralplatform_classification WHERE page_id = {page_id};"):
        run_delete(f"DELETE FROM neuralplatform_classification WHERE page_id = {page_id};")
    run_insert(f"INSERT INTO neuralplatform_classification(classDefinition_id, page_id) VALUES ({class_id}, {page_id});")
    update_object_by_key('page', 'id', page_id, {'tagged': True})

def insert_manualStep(result, request_id):
    query = f"INSERT INTO neuralplatform_manualstep(status, result, request_id) " + \
            f"VALUES ('pending', '{json.dumps(result)}', {request_id});"
    return run_insert(query)

######## DEPRECATED ########

def get_user_pk_by_username(username):
	query = f"SELECT id FROM automlapp_user WHERE username = '{username}';"
	pk = run_select(query)[0]
	return pk

def get_user_pk_by_username_password(username, password):
	pk = -1
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = "SELECT id FROM automlapp_user WHERE username = \"" + username + "\" AND password = \"" + password + "\""
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		pk = response[0]
	except:
		print("get_user_pk_by_username_password - ERROR")
	finally:
		db.close()
	return pk

def get_project_pk_by_user_pk_project_name(user_pk, project_name):
	pk = -1
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT id FROM automlapp_project WHERE user_id = {user_pk} AND proj_name = "{project_name}";'
		# query = "SELECT id FROM automlapp_project WHERE user_id = " + str(user_pk) + " AND proj_name = \"" + project_name + "\";"
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		pk = response[0]
	except Exception as e:
		print("get_project_pk_by_user_pk_project_name - ERROR " + str(e))
	finally:
		db.close()
	return pk

def get_project_id_by_model_id(model_id):
	project_id = -1
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f"SELECT project_id FROM automlapp_modelversion WHERE id = {model_id};"
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		if response[0] != None:
			project_id = int(response[0])
	except Exception as e:
		print("get_project_id_by_model_id - ERROR " + str(e))
	finally:
		db.close()
	return project_id

def get_project_name_by_project_pk(project_pk):
	name = ""
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f"SELECT proj_name FROM automlapp_project WHERE id = {project_pk};"
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		name = str(response[0])
	except Exception as e:
		print("get_project_name_by_project_pk : ERROR : " + str(e))
	finally:
		db.close()
	return name

def get_document_count_by_project_id_and_label(project_id, label):
	count = 0
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f"SELECT COUNT(*) FROM automlapp_file WHERE project_id = {project_id} AND label = {label};"
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		count = int(response[0])
	except Exception as e:
		print("get_document_count_by_project_id_and_label : ERROR : " + str(e))
	finally:
		db.close()
	return count

def insert_files_to_rds(paths, project_name, user_pk):
	inserted = False
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		project_pk = get_project_pk_by_user_pk_project_name(user_pk, project_name)
		cursor = db.cursor()

		for file_path in paths:
			file_name, file_ext = os.path.splitext(os.path.basename(file_path))
			file_ext = file_ext.replace('.','')
			query = '''INSERT INTO automlapp_file(file_type, file_name, tag_manual, uri, project_id, trained)
								VALUES("{}","{}",{},"{}",{},{})'''.format(file_ext, file_name, 0, file_path, project_pk, 0)
			cursor.execute(query)
			print(f"inserted file {file_path} to RDS")
		db.commit()
	except mysql.IntegrityError:
		pass
	except Exception as e:
		print("insert_file - ERROR " + str(e))
	finally:
		db.close()
	return True

def update_job_result(job_id, result):
	result = round(result, 2)
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)

		query = f'UPDATE automlapp_job SET result = {result} where id = {job_id};'
		cursor = db.cursor()
		cursor.execute(query)
		db.commit()
	except Exception as e:
		print("update_job_result - ERROR " + str(e))
	finally:
		db.close()

def get_png_uri_from_page_id(page_id):
	uri = None
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f"SELECT png_uri FROM automlapp_page WHERE id = {page_id};"
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		uri = response[0]
	except Exception as e:
		print("get_png_uri_from_page_id - ERROR " + str(e))
	finally:
		db.close()
	return uri

def get_file_uri_and_label_from_id(file_id):
	uri = label = None
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f"SELECT uri, label FROM automlapp_file WHERE id = {file_id};"
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		uri = response[0]
		label = response[1]
	except Exception as e:
		print("get_file_uri_from_id : ERROR : " + str(e))
	finally:
		db.close()
	return uri, label

def get_trained_model_path(project_id):
	path = None
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f"SELECT trained_model_path FROM automlapp_model WHERE project_id = {project_id};"
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		path = response[0]
	except Exception as e:
		print("get_trained_model_path : ERROR : " + str(e))
	finally:
		db.close()
	return path

def get_npages_to_preprocess_for_project(project_id):
	npages = 0
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT SUM(npages) FROM automlapp_file WHERE project_id = {project_id} AND trained = 0 AND preprocessed = 0;'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		if response[0] != None:
			npages = response[0]
	except Exception as e:
		print("get_npages_to_preprocess_for_project : ERROR : " + str(e))
	finally:
		db.close()
	return int(npages)

def get_npages_of_file(file_id):
	npages = 0
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT npages FROM automlapp_file WHERE id = {file_id};'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		if response[0] != None:
			npages = response[0]
	except Exception as e:
		print("get_npages_of_file : ERROR : " + str(e))
	finally:
		db.close()
	return int(npages)

def all_pages_processed_for_file_ids(file_ids):
	all_processed = False
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query1 = f'SELECT SUM(npages) FROM automlapp_file WHERE id IN ({str(file_ids)[1:-1]});'
		query2 = f'SELECT COUNT(*) FROM automlapp_page WHERE file_id IN ({str(file_ids)[1:-1]}) AND ocr_uri IS NOT NULL;'
		cursor = db.cursor()
		cursor.execute(query1)
		result1 = cursor.fetchone()[0]
		if result1:
			total_pages = int(result1)
			cursor.execute(query2)
			result2 = cursor.fetchone()[0]
			if result2:
				processed_pages = int(result2)
				all_processed = total_pages == processed_pages
	except Exception as e:
		print("all_pages_processed_for_file_ids : ERROR : " + str(e))
	finally:
		db.close()
	return all_processed

def set_files_preprocessed(file_ids):
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'UPDATE automlapp_file SET preprocessed = 1 WHERE id IN ({str(file_ids)[1:-1]});'
		cursor = db.cursor()
		cursor.execute(query)
		db.commit()
	except Exception as e:
		print("set_files_preprocessed : ERROR : " + str(e))
	finally:
		db.close()

def get_file_ids_to_preprocess_for_project(project_id):
	ids = []
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT id FROM automlapp_file WHERE project_id = {project_id} AND trained = 0 AND preprocessed = 0;'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchall()
		for row in response:
			ids.append(row[0])
	except Exception as e:
		print("get_file_ids_to_preprocess_for_project : ERROR : " + str(e))
	finally:
		db.close()
	return ids

def get_file_ids_completely_processed_for_project(project_id):
	ids = []
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT DISTINCT file_id FROM automlapp_page pg WHERE file_id IN (SELECT id FROM automlapp_file WHERE project_id = {project_id}) AND NOT EXISTS (SELECT * FROM automlapp_page pg2 WHERE pg.file_id = pg2.file_id AND ocr_uri IS null);'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchall()
		for row in response:
			ids.append(row[0])
	except Exception as e:
		print("get_file_ids_completely_processed_for_project : ERROR : " + str(e))
	finally:
		db.close()
	return ids

def get_pages_of_files(file_ids):
	ids = []
	png_uris = []
	ocr_uris = []
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT label, png_uri, ocr_uri FROM automlapp_page WHERE file_id IN ({str(file_ids)[1:-1]});'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchall()
		for row in response:
			ids.append(row[0])
			png_uris.append(row[1])
			ocr_uris.append(row[2])
	except Exception as e:
		print("get_pages_of_files : ERROR : " + str(e))
	finally:
		db.close()
	return ids, png_uris, ocr_uris

def delete_file_from_rds(username, file_uri):

	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)

		query = f"DELETE FROM automlapp_file WHERE uri ='{file_uri}';"
		cursor = db.cursor()
		cursor.execute(query)
		db.commit()
	except Exception as e:
		print("delete_file_from_rds : ERROR : " + str(e))
	finally:
		db.close()

def update_ocr_uri_by_page_id(page_id, ocr_uri):
	print(f"update_ocr_uri_by_page_id : INFO : page_id = {page_id}, ocr_uri = {ocr_uri}")
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)

		query = f'UPDATE automlapp_page SET ocr_uri = "{ocr_uri}" where id = {page_id};'
		cursor = db.cursor()
		cursor.execute(query)
		db.commit()
	except Exception as e:
		print("update_ocr_uri_by_page_id : ERROR : " + str(e))
	finally:
		db.close()

def create_training_job(model_id: int, output_path: str) -> int:
	pk = -1
	try:
		project_id = get_project_id_by_model_id(model_id)
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)

		query = f'INSERT INTO automlapp_job(status, model_id, result, job_type, output_path, project_id) VALUES ("CREATED", {model_id}, 0, "TRAIN", "{output_path}", {project_id});'
		cursor = db.cursor()
		cursor.execute(query)
		pk = cursor.lastrowid
		db.commit()
	except mysql.IntegrityError:
		pass
	except Exception as e:
		print("create_training_job : ERROR : " + str(e))
	finally:
		db.close()
	return pk

def get_file_ids_preprocessed_untrained(project_id):
	ids = []
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT id FROM automlapp_file WHERE project_id = {project_id} AND trained = 0 AND preprocessed = 1;'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchall()
		for row in response:
			ids.append(int(row[0]))
	except Exception as e:
		print("get_file_ids_preprocessed_untrained : ERROR : " + str(e))
	finally:
		db.close()
	return ids

def get_model_hyperparams(model_id):
	hyperparams = {}
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT hyperparams FROM automlapp_modelversion WHERE id = {model_id};'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		if response[0] != None:
			hyperparams_json = json.loads(response[0])
			for key in hyperparams_json:
				hyperparams[key.upper()] = hyperparams_json[key]
	except Exception as e:
		print("get_model_hyperparams : ERROR : " + str(e))
	finally:
		db.close()
	return hyperparams

def get_trained_model_path(model_id):
	path = ''
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT trained_model_path FROM automlapp_modelversion WHERE id = {model_id};'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		if response[0] != None:
			path = response[0]
	except Exception as e:
		print("get_trained_model_path : ERROR : " + str(e))
	finally:
		db.close()
	return path

def get_trainings_for_project(project_id):
	trainings = 0
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT trainings FROM automlapp_project WHERE id = {project_id};'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		if response[0] != None:
			trainings = int(response[0])
	except Exception as e:
		print("get_trainings_for_project : ERROR : " + str(e))
	finally:
		db.close()
	return trainings

def get_raw_model_name_of_model_version(model_id):
	raw_model_name = ''
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query1 = f'SELECT raw_model_id FROM automlapp_modelversion WHERE id = {model_id};'
		cursor = db.cursor()
		cursor.execute(query1)
		response = cursor.fetchone()
		raw_model_id = int(response[0])
		query2 = f'SELECT name FROM automlapp_rawmodel WHERE id = {raw_model_id};'
		cursor.execute(query2)
		response = cursor.fetchone()
		raw_model_name = str(response[0])
	except Exception as e:
		print("get_raw_model_name_of_model_version : ERROR : " + str(e))
	finally:
		db.close()
	return raw_model_name

def get_model_ids_for_project(project_id):
	ids = []
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT id FROM automlapp_modelversion WHERE project_id = {project_id};'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchall()
		for row in response:
			ids.append(int(row[0]))
	except Exception as e:
		print("get_model_ids_for_project : ERROR : " + str(e))
	finally:
		db.close()
	return ids

def get_best_model_id_and_accuracy_for_project(project_id):
	id = -1
	ac = 0.0
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT id, accuracy FROM automlapp_modelversion WHERE project_id = {project_id};'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchall()
		for row in response:
			next_id = int(row[0])
			next_ac = float(row[1])
			if next_ac >= ac:
				id = next_id
				ac = next_ac
	except Exception as e:
		print("get_best_model_for_project : ERROR : " + str(e))
	finally:
		db.close()
	return id, ac

def get_training_image_tag_for_model_version(model_id):
	training_image = ''
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		cursor = db.cursor()
		query1 = f'select raw_model_id from automlapp_modelversion WHERE id = {model_id};'
		cursor.execute(query1)
		response = cursor.fetchone()
		raw_model_id = int(response[0])
		query2 = f'SELECT tag FROM automlapp_trainingimage WHERE id IN (SELECT training_image_id FROM automlapp_rawmodel WHERE id = {raw_model_id});'
		cursor.execute(query2)
		response = cursor.fetchone()
		training_image = str(response[0])
	except Exception as e:
		print("get_training_image_tag_for_model_version : ERROR : " + str(e))
	finally:
		db.close()
	return training_image

def update_trained_model_path(model_id, trained_model_path):
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)

		query = f'UPDATE automlapp_modelversion SET trained_model_path = "{trained_model_path}" where id = {model_id};'
		cursor = db.cursor()
		cursor.execute(query)
		db.commit()
	except Exception as e:
		print("update_trained_model_path : ERROR : " + str(e))
	finally:
		db.close()

def tag_file(file_id, label):
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)

		query = f'UPDATE automlapp_file SET label = {label} where id = {file_id};'
		cursor = db.cursor()
		cursor.execute(query)
		db.commit()
	except Exception as e:
		print("tag_file : ERROR : " + str(e))
	finally:
		db.close()

def get_last_used_port():
	last_used_port = 79
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f"SELECT MAX(lb_port) FROM automlapp_project;"
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		if response:
			last_used_port = int(response[0])
	except Exception as e:
		print("get_last_used_port : ERROR : " + str(e))
	finally:
		db.close()
	return last_used_port

def get_project_files(project_pk):
	uris = []
	labels = []
	npages = []
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f'SELECT uri, label, npages FROM automlapp_file WHERE project_id = {project_pk};'
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchall()
		for row in response:
			uris.append(row[0])
			labels.append(row[1])
			npages.append(int(row[2]))
	except Exception as e:
		print("get_project_files : ERROR : " + str(e))
	finally:
		db.close()
	return uris, labels, npages

def get_project_confianza(project_pk):
	confianza = 0
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f"SELECT confianza FROM automlapp_project WHERE id = {project_pk};"
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		if response:
			confianza = int(response[0])
	except Exception as e:
		print("get_project_confianza : ERROR : " + str(e))
	finally:
		db.close()
	return confianza

def get_project_port(project_pk):
	port = 80
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f"SELECT lb_port FROM automlapp_project WHERE id = {project_pk};"
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		if response:
			port = int(response[0])
	except Exception as e:
		print("get_project_port : ERROR : " + str(e))
	finally:
		db.close()
	return port

def get_cluster_name_of_project(project_pk):
	cluster_name = ""
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)
		query = f"SELECT cluster_name FROM automlapp_project WHERE id = {project_pk};"
		cursor = db.cursor()
		cursor.execute(query)
		response = cursor.fetchone()
		if response:
			cluster_name = str(response[0])
	except Exception as e:
		print("v : ERROR : " + str(e))
	finally:
		db.close()
	return cluster_name



import boto3
import base64
import hmac
import hashlib
from .automl import AWS_ACC_KEY_ID, AWS_SEC_ACC_KEY, USER_POOL_ID, CLIENT_ID, CLIENT_SECRET, AWS_REGION_NAME

client_cognito = boto3.client('cognito-idp',
						aws_access_key_id=AWS_ACC_KEY_ID,
						aws_secret_access_key=AWS_SEC_ACC_KEY,
						region_name=AWS_REGION_NAME)


def get_secret_hash(username):
	msg = username + CLIENT_ID
	dig = hmac.new(str(CLIENT_SECRET).encode('utf-8'),
		msg = str(msg).encode('utf-8'), digestmod=hashlib.sha256).digest()
	d2 = base64.b64encode(dig).decode()
	return d2

def sign_up_user(username, password, email):
	try:
		resp = client_cognito.sign_up(
			ClientId=CLIENT_ID,
			SecretHash=get_secret_hash(username),
			Username=username,
			Password=password,
			UserAttributes=[
			{
				'Name': "email",
				'Value': email
			}
			],
			ValidationData=[
				{
				'Name': "email",
				'Value': email
			},
			{
				'Name': "custom:username",
				'Value': username
			}
	])


	except client_cognito.exceptions.UsernameExistsException as e:
		return {"error": False,
				"success": True,
				"message": "This username already exists",
				"data": None}
	except client_cognito.exceptions.InvalidPasswordException as e:

		return {"error": False,
				"success": True,
				"message": "Password should have Caps, Special chars, Numbers",
				"data": None}
	except client_cognito.exceptions.UserLambdaValidationException as e:
		return {"error": False,
				"success": True,
				"message": "Email already exists",
				"data": None}

	except Exception as e:
		return {"error": False,
				"success": True,
				"message": str(e),
				"data": None}

	return {"error": False,
			"success": True,
			"message": "Please confirm your signup, check Email for validation code",
			"data": None}

def confirm_sign_up(username, code):
	try:
		response = client_cognito.confirm_sign_up(
		ClientId=CLIENT_ID,
		SecretHash=get_secret_hash(username),
		Username=username,
		ConfirmationCode=code,
		ForceAliasCreation=False,
	   )
	except client_cognito.exceptions.UserNotFoundException:
		return {"error": True, "success": False, "message": "Username doesnt exists"}
		# return event
	except client_cognito.exceptions.CodeMismatchException:
		return {"error": True, "success": False, "message": "Invalid Verification code"}

	except client_cognito.exceptions.NotAuthorizedException:
		return {"error": True, "success": False, "message": "User is already confirmed"}

	except Exception as e:
		return {"error": True, "success": False, "message": f"Unknown error {e.__str__()} "}

	# return event
	return {"error": False, "success": True, "message": "Username confirmed"}

def initiate_auth(username, password):
	secret_hash = get_secret_hash(username)
	try:
		resp = client_cognito.initiate_auth(
			# AuthFlow='USER_SRP_AUTH'|'REFRESH_TOKEN_AUTH'|'REFRESH_TOKEN'|'CUSTOM_AUTH'|'ADMIN_NO_SRP_AUTH'|'USER_PASSWORD_AUTH'|'ADMIN_USER_PASSWORD_AUTH',
			AuthFlow='USER_PASSWORD_AUTH',
			AuthParameters={
					'USERNAME': username,
					'SECRET_HASH': secret_hash,
					'PASSWORD': password
			},
			ClientId=CLIENT_ID,
		)

	except client_cognito.exceptions.NotAuthorizedException:
		return None, "The username or password is incorrect"
	except client_cognito.exceptions.UserNotConfirmedException:
		return None, "User is not confirmed"
	except Exception as e:
		return None, e.__str__()
	return resp, None

def signin_user(username, password):
	resp, msg = initiate_auth(username, password)
	if msg != None:
		return {'message': msg,
			"error": True,
			"success": False,
			"data": None}

	if resp.get("AuthenticationResult"):
		return {'message': "success",
			"error": False,
			"success": True,
			"data": {
			"id_token": resp["AuthenticationResult"]["IdToken"],
			"refresh_token": resp["AuthenticationResult"]["RefreshToken"],
			"access_token": resp["AuthenticationResult"]["AccessToken"],
			"expires_in": resp["AuthenticationResult"]["ExpiresIn"],
			"token_type": resp["AuthenticationResult"]["TokenType"]
		  }}
	else: #this code block is relevant only when MFA is enabled
		return {"error": True,
			"success": False,
			"data": None, "message": None}



import boto3
from datetime import datetime, timedelta
from .automl import AWS_ACC_KEY_ID, AWS_SEC_ACC_KEY, AWS_REGION_NAME
import json

client_cw = boto3.client('cloudwatch',
						aws_access_key_id=AWS_ACC_KEY_ID,
						aws_secret_access_key=AWS_SEC_ACC_KEY,
						region_name=AWS_REGION_NAME)

# usefeul parameters:
day = 86400
hour = 3600
minute = 60

delta_days = 3
start_time = datetime.now() - timedelta(days=delta_days)
end_time = datetime.now()

def update_metric_info(metricName, metricValue):
	client_cw.put_metric_data(
		MetricData = [
			{
				'MetricName': metricName,
				'Dimensions': [
					{
					'Name': 'Instance name',
					'Value': 'asdsadsd'
					}
				],
				'Unit': 'None',
				'Value': metricValue
			},
		],
		Namespace = 'OpsNamespace'
	)

def get_metric_statistics():
	response = client_cw.get_metric_statistics(
		Namespace='AWS/RDS',
		Dimensions=[
			{
				'Name': 'DBInstanceIdentifier',
				'Value': 'aazp6ut01gnv68'
			}
		],
		MetricName='ReadIOPS',
		StartTime=start_time,
		EndTime=end_time,
		Period=hour*2,
		Statistics=[
			'Average'
		]
	)

	# print(str(response).replace('\'', '\"'))
	return response

def get_metrics_rds():
	response = client_cw.get_metric_widget_image(
	MetricWidget= '''{
			"start": "-PT5H",
			"yAxis": {
				"left": {
					"min": 0,
					"max": 700000000
				}
			},
			"metrics": [
				[
					"AWS/RDS",
					"FreeableMemory",
					"DBInstanceIdentifier",
					"aazp6ut01gnv68"
				]
			]
		}'''
	)
	png_text = response['MetricWidgetImage']
	# return io.BytesIO(png_text)
	return png_text

def get_metric_data_rds():
	response = client_cw.get_metric_data(
		MetricDataQueries=[
			{
				'Id':'myId',
				'MetricStat': {
					'Metric': {
						'Namespace':'AWS/RDS',
						'MetricName': 'ReadIOPS',
						'Dimensions': [
							{
								'Name': 'DBInstanceIdentifier',
								'Value': 'aazp6ut01gnv68'
							},
						]
					},
					'Period': hour*2,
					'Stat': 'Average'
				},
			},
		],
		StartTime=start_time,
		EndTime=end_time,
		ScanBy='TimestampAscending',
	)

	# print(str(response).replace('\'', '\"'))
	return response

def get_metric_widget_image_rds():
	metrics = get_metric_data_rds()
	# timestamps = metrics['MetricDataResults'][0]['Timestamps']
	values = metrics['MetricDataResults'][0]['Values']
	content = {
			"start": "-PT"+ str(delta_days*24) +"H",
			"yAxis": {
				"left": {
					"min": min(values)*0.8,
					"max": max(values)*1.2

}			},
			"metrics": [
				[
					"AWS/RDS",
					"ReadIOPS",
					"DBInstanceIdentifier",
					"aazp6ut01gnv68",
					{ "stat": "Average" }
				]
			],
			"period": 60,
			"timezone": "+0100"
		}
	response = client_cw.get_metric_widget_image(
		MetricWidget= json.dumps(content)
	)

	png_text = response['MetricWidgetImage']
	# image = Image.open(io.BytesIO(png_text))
	# image.show()
	return png_text

def get_metric_statistics_s3():
	response = client_cw.get_metric_statistics(
		Namespace='AWS/S3',
		Dimensions=[
			{
				'Name': 'BucketName',
				'Value': 'elasticbeanstalk-us-west-2-749868801319'
			},
			{
				'Name': 'StorageType',
				'Value': 'StandardStorage'
			}
		],
		MetricName='BucketSizeBytes',
		StartTime=start_time,
		EndTime=end_time,
		Period=day,
		Statistics=[
			'Maximum'
		],
		Unit='Bytes'
	)
	# print(str(response).replace('\'', '\"'))
	return response

def get_metric_data_s3():
	response = client_cw.get_metric_data(
		MetricDataQueries=[
			{
				'Id':'myId',
				'MetricStat': {
					'Metric': {
						'Namespace':'AWS/S3',
						'MetricName': 'BucketSizeBytes',
						'Dimensions': [
							{
								'Name': 'BucketName',
								'Value': 'elasticbeanstalk-us-west-2-749868801319'
							},
							{
								'Name': 'StorageType',
								'Value': 'StandardStorage'
							},
						]
					},
					'Period': hour*2,
					'Stat': 'Maximum',
					'Unit': 'Bytes'
				},
			},
		],
		StartTime=start_time,
		EndTime=end_time,
		ScanBy='TimestampAscending',
	)
	# print(str(response).replace('\'', '\"'))
	return response

def get_metric_widget_image_s3():
	metrics = get_metric_data_s3()
	# timestamps = metrics['MetricDataResults'][0]['Timestamps']
	values = metrics['MetricDataResults'][0]['Values']
	content = {
			"start": "-PT"+ str(delta_days*24) +"H",
			"yAxis": {
				"left": {
					"min": min(values)*0.8,
					"max": max(values)*1.2
				}
			},
			"metrics": [
				[
					"AWS/S3",
					"BucketSizeBytes",
					"BucketName",
					"elasticbeanstalk-us-west-2-749868801319",
					"StorageType",
					"StandardStorage"
				]
			],
			"period": 60,
			"timezone": "+0100"
		}
	response = client_cw.get_metric_widget_image(
		MetricWidget= json.dumps(content)
	)
	png_text = response['MetricWidgetImage']
	return png_text

# Para preguntarle al usuario por una fecha concreta, Ejemplo:
# date_entry = input('Enter a date in DD-MM-YYYY format')
# day, month, year = map(int, date_entry.split('-'))
# date1 = datetime.date(year, month, day)



import os
import json
from os.path import expanduser

AWS_ACC_KEY_ID = ''
AWS_SEC_ACC_KEY = ''
AWS_REGION_NAME = ''
USER_POOL_ID = ''
CLIENT_ID = ''
CLIENT_SECRET = ''
BD_HOST = ''
BD_PASS = ''
BD_DATABASE = ''
BD_USER = ''

def init():

	try:
		config_file_path = os.environ['AUTOMLAPI_CONFIG_FILE_PATH']
	except Exception as e:
		config_file_path = os.path.join(expanduser("~"),'automlapi.config')

	with open(config_file_path, 'r') as f:
		config_string = f.read()
		j = json.loads(config_string)
		global AWS_ACC_KEY_ID
		AWS_ACC_KEY_ID = j['AWS_ACC_KEY_ID']
		global AWS_SEC_ACC_KEY
		AWS_SEC_ACC_KEY = j['AWS_SEC_ACC_KEY']
		global AWS_REGION_NAME
		AWS_REGION_NAME = j['AWS_REGION_NAME']
		global USER_POOL_ID
		USER_POOL_ID = j['USER_POOL_ID']
		global CLIENT_ID
		CLIENT_ID = j['CLIENT_ID']
		global CLIENT_SECRET
		CLIENT_SECRET = j['CLIENT_SECRET']
		global BD_HOST
		BD_HOST = j['BD_HOST']
		global BD_PASS
		BD_PASS = j['BD_PASS']
		global BD_DATABASE
		BD_DATABASE = j['BD_DATABASE']
		global BD_USER
		BD_USER = j['BD_USER']



import boto3
from .automl import AWS_ACC_KEY_ID, AWS_SEC_ACC_KEY, AWS_REGION_NAME
import time
from botocore.exceptions import ClientError

client_ecs = boto3.client('ecs',
						aws_access_key_id=AWS_ACC_KEY_ID,
						aws_secret_access_key=AWS_SEC_ACC_KEY,
						region_name=AWS_REGION_NAME)

def get_servicesArns_of_cluster(cluster_name):
	while True:
		try:
			return client_ecs.list_services(cluster=cluster_name)['serviceArns']
		except ClientError:
			time.sleep(1)
			pass

def get_service(cluster_name, service_name):
	while True:
		try:
			return client_ecs.describe_services(cluster=cluster_name, services=[service_name])['services'][0]
		except ClientError:
			time.sleep(1)
			pass

def get_max_instances(cluster_name, service_name):
	# TODO: IMPLEMENT
	return 5

def update_service_desiredCount(cluster_name, service_name, desiredCount):
	while True:
		try:
			client_ecs.update_service(cluster=cluster_name, service=service_name, desiredCount=desiredCount)
			break
		except ClientError:
			time.sleep(1)
			pass


### DEPRECATED ######
def update_flask_service_instances(service, num_instances, cluster_name):
	num_instances = min(num_instances, 20)
	num_instances = max(num_instances, 0)
	print(f"update_flask_service_instances : INFO : Requesting {num_instances} for service {service}...")
	try:
		response = client_ecs.update_service(
			cluster=cluster_name,
			service=service,
			desiredCount=int(num_instances),
			forceNewDeployment=True
		)
	except Exception:
		return False
	return True

def get_service_instaces_status(cluster, service_name):
	print(f"get_service_instaces_status : INFO : Getting info of service {service_name} (cluster {cluster})...")
	try:
		response = client_ecs.describe_services(cluster=cluster, services=[service_name])
		service = response['services'][0]
		desired = service['desiredCount']
		running = service['runningCount']
		return desired, running
	except:
		return 0, 0

def services_ready(cluster, service_list):
	response = client_ecs.describe_services(cluster=cluster, services=service_list)
	services = response['services']
	for service in services:
		name    = service['serviceName']
		desired = service['desiredCount']
		running = service['runningCount']
		print(f"services_ready : INFO : Service {name}: Desired = {desired}, Running = {running}")
		if desired != running:
			return False
	return True

def create_cluster(cluster_name):
	response = client_ecs.create_cluster(clusterName=cluster_name)

def create_task_definition(task_definition_name, ecr_image):
	response = client_ecs.register_task_definition(
	    family=task_definition_name,
	    taskRoleArn='arn:aws:iam::749868801319:role/ecsTaskExecutionRole',
	    executionRoleArn='arn:aws:iam::749868801319:role/ecsTaskExecutionRole',
	    networkMode='bridge',
	    containerDefinitions=[
	        {
	            'name': 'CONTAINER',
	            'image': ecr_image,
	            'cpu': 0,
	            'memory': 6144,
	            'portMappings': [
	                {
	                    'containerPort': 5000,
	                    'hostPort': 5000,
	                    'protocol': 'tcp'
	                },
					{
	                    'containerPort': 5555,
	                    'hostPort': 5555,
	                    'protocol': 'tcp'
	                }
	            ],
	            'essential': True,
	            'workingDirectory': '/'
	        },
	    ],
	    requiresCompatibilities=[
	        'EC2',
	    ],
	    cpu='2000',
	    memory='6144'
	)
	if int(response['ResponseMetadata']['HTTPStatusCode']) == 200:
		print(f'create_task_definition : INFO : Created TaskDefinition {task_definition_name}!')
		return str(response['taskDefinition']['taskDefinitionArn'])
	else:
		print(f'create_task_definition : ERROR : Could not create TaskDefinition {task_definition_name}!')
		return False

def create_service(cluster, serviceName, taskDefinition, targetGroupArn):
	response = client_ecs.create_service(
	    cluster=cluster,
	    serviceName=serviceName,
	    taskDefinition=taskDefinition,
	    loadBalancers=[
	        {
	            'targetGroupArn': targetGroupArn,
	            'containerName': 'CONTAINER',
	            'containerPort': 5000
	        },
	    ],
	    desiredCount=0,
	    launchType='EC2',
	    role='arn:aws:iam::749868801319:role/ecsServiceRole',
	    deploymentConfiguration={
	        'maximumPercent': 200,
	        'minimumHealthyPercent': 0
	    },
	    placementStrategy=[
	        {
	            'type': 'binpack',
	            'field': 'memory'
	        },
	    ],
	    healthCheckGracePeriodSeconds=2147483646,
	    schedulingStrategy='REPLICA',
	    enableECSManagedTags=True
	)
	return int(response['ResponseMetadata']['HTTPStatusCode']) == 200



import boto3
import zipfile
import os
from tqdm import tqdm
import copy
import json
import botocore.exceptions
import io
from botocore.exceptions import ClientError
from PyPDF2 import PdfFileReader
from .automl import AWS_ACC_KEY_ID, AWS_SEC_ACC_KEY, AWS_REGION_NAME


client_s3 = boto3.client('s3',
						aws_access_key_id=AWS_ACC_KEY_ID,
						aws_secret_access_key=AWS_SEC_ACC_KEY,
						region_name=AWS_REGION_NAME)

resource_s3 = boto3.resource('s3',
						aws_access_key_id=AWS_ACC_KEY_ID,
						aws_secret_access_key=AWS_SEC_ACC_KEY,
						region_name=AWS_REGION_NAME)

######## UPDATED ########

def upload_file(bucketName, file, path):
	# file must be opened in 'rb'
	try:
		client_s3.upload_fileobj(file, bucketName, path)
		return True
	except Exception as e:
		print(f"upload_file : ERROR : {e}")
		return False

def download_file(bucketName, s3_path, local_path):
	try:
		bucket = resource_s3.Bucket(bucketName)
		bucket.download_file(s3_path, local_path)
		return True
	except Exception as e:
		print(f"download_file : ERROR : {e}")
		return False

####### DEPRECATED #######

def write_file(filepath, content):
	bucketName = S3_BUCKET
	bucket = resource_s3.Bucket(bucketName)
	try:
		object = bucket.Object(filepath)
		object.put(Body=content)
	except ClientError as e:
		print(str(e))

def delete_file_from_s3(username, file_uri):
	bname = get_bucketname_by_username(username)
	try:
		resource_s3.Object(bname, file_uri).delete()
	except Exception as e:
		print(f"delete_file_from_s3 : ERROR : {e}")

def upload_fileobj(file, bucket, key):
	client_s3.upload_fileobj(file, bucket, key)

def upload_document_to_s3(file, username, project_name):
	# TODO: Count npages of each document!!!
	s3_data_folder = f'user_data/{username}/{project_name}/data/raw_files/'
	filename, extension = os.path.splitext(file.name)

	new_pdfs = {}
	known_tags = {}
	extra_tags = {}

	if extension == '.pdf':
		s3_pdf_path = os.path.join(s3_data_folder, os.path.basename(file.name))
		content = file.read()
		file_copy = io.BytesIO(content)
		file_copy2 = io.BytesIO(content)
		npages = PdfFileReader(file_copy).getNumPages()
		print(f"upload_document_to_s3 : INFO : uploading {s3_pdf_path}...")
		private_upload_file(file_copy2, username, s3_pdf_path)
		new_pdfs[s3_pdf_path] = {}
		new_pdfs[s3_pdf_path]['label'] = -1
		new_pdfs[s3_pdf_path]['npages'] = npages
	else:
		classes = get_project_classes(username, project_name)
		if extension == '.tag':
			print(f"upload_document_to_s3 : INFO : a .tag file was uploaded!")
			content = file.read().decode('utf-8').lower()
			try:
				label = classes.index(content)
				print(f"upload_document_to_s3 : INFO : label = {label}")
				s3_pdf_path = os.path.join(s3_data_folder, os.path.basename(file.name))
				s3_pdf_path = s3_pdf_path.replace('.tag', '.pdf')
				extra_tags[s3_pdf_path] = label
			except ValueError:
				# si nos suben un unico fichero tag, con la clase mal puesta no hacemos nada...
				pass

		elif extension == '.zip':
			unzipped = zipfile.ZipFile(file)
			file_names = unzipped.namelist()
			for file_name in file_names:
				_, suf_file_ext = os.path.splitext(file_name)
				sub_file = unzipped.open(file_name)
				if suf_file_ext == '.pdf':
					s3_pdf_path = os.path.join(s3_data_folder, os.path.basename(file_name))
					content = sub_file.read()
					file_copy = io.BytesIO(content)
					file_copy2 = io.BytesIO(content)
					npages = PdfFileReader(file_copy).getNumPages()
					private_upload_file(file_copy2, username, s3_pdf_path)
					new_pdfs[s3_pdf_path] = {}
					new_pdfs[s3_pdf_path]['label'] = -1
					new_pdfs[s3_pdf_path]['npages'] = npages
				elif suf_file_ext == '.tag':
					s3_pdf_path = os.path.join(s3_data_folder, os.path.basename(file_name))
					s3_pdf_path= s3_pdf_path.replace('.tag', '.pdf')
					content = unzipped.read(file_name).decode('utf-8').lower()
					label = classes.index(content)
					known_tags[s3_pdf_path] = label

			# Check for matches between .pdf & .tag
			for entry in known_tags:
				if entry in new_pdfs:
					new_pdfs[entry]['label'] = known_tags[entry]
				else:
					extra_tags[entry] = known_tags[entry]

	return new_pdfs, extra_tags

def get_file(username: str,
             path: str) -> bytes:
    """
    """
    # TODO: use username's specific bucket
    bucketName = "elasticbeanstalk-us-west-2-749868801319"
    return client_s3.get_object(Bucket=bucketName, Key=path)

def download_file_from_s3(username, objectname, local_file):
	try:
		bucketName = get_bucketname_by_username(username)
		bucket = resource_s3.Bucket(bucketName)
		bucket.download_file(objectname, local_file)
		return True
	except Exception as e:
		print(f"download_file_from_s3 : ERROR : {e}")
		return False

def download_directory(bucketName, s3_dirname, local_dirname):
	if not os.path.exists(local_dirname):
		os.makedirs(local_dirname)
	bucket = resource_s3.Bucket(bucketName)
	for object in tqdm(bucket.objects.filter(Prefix = s3_dirname)):
		filename = os.path.basename(object.key)
		if filename != '':
			local_file = os.path.join(local_dirname, filename)
			bucket.download_file(object.key, local_file)

def get_bucketname_by_username(username):
	# TODO: Hacer una query a la tabla user para ver que bucket tiene asociado
	return 'elasticbeanstalk-us-west-2-749868801319'


def private_upload_file(file, username, s3_path):
	try:
		bname = get_bucketname_by_username(username)
		client_s3.upload_fileobj(file, bname, s3_path)
		return True
	except Exception as e:
		print("private_upload_file : ERROR : " + str(e))
		return False

def get_folder_size(bucket, prefix):
	# RETURNS FOLDER SIZE IN MB
	# AWS CONSOLE RETURNS SIZE IN MiB
	# 1 MiB = 1,04858 MB
	total_size = 0
	for obj in resource_s3.Bucket(bucket).objects.filter(Prefix=prefix):
		total_size += obj.size
		return total_size

def upload_folder_as_zip(local_folder, s3_folder, username):

	def zipdir(path, ziph):
		for root, dirs, files in os.walk(path):
			for file in files:
				ziph.write(os.path.join(root, file))

	zip_name = f'{local_folder}.zip'
	with zipfile.ZipFile(zip_name, 'w', zipfile.ZIP_DEFLATED) as zipf:
		zipdir(local_folder, zipf)
	zipf = open(zip_name, 'rb')
	objectname = s3_folder + '/' + zip_name
	upload_file(zipf, username, objectname)

def update_checkpoints(local_chkpts_dir, s3_chkpts_dir, username):
	for item in os.listdir(local_chkpts_dir):
		with open(os.path.join(local_chkpts_dir, item), 'rb') as f:
			objectname = s3_chkpts_dir + '/' + item
			upload_file(f, username, objectname)

def get_project_classes(username, project_name):
	objectname = f'user_data/{username}/{project_name}/data/index.txt'
	local_file = f'index.txt'
	download_file_from_s3(username, objectname, local_file)
	return [x.lower() for x in open(local_file, 'r').read().split(',')]



import pymysql as mysql
import os
import hashlib
import json


BD_HOST = 'aaf8asdx5lx07e.cqioz7mocjad.eu-west-3.rds.amazonaws.com'
BD_PASS = 'Sca3Zp5#g'
BD_DATABASE = 'ebdb'
BD_USER = 'dbadmin'

def run_exists(query):

    exists = False

    try:
        db = mysql.connect(host=BD_HOST,
                database=BD_DATABASE,
                user=BD_USER,
                password=BD_PASS)
        cursor = db.cursor()
        cursor.execute(query)
        response = cursor.fetchall()
        exists = len(response) > 0
    except Exception as e:
        print(f"run_exists - ERROR - {e}")
    finally:
        db.close()

    return exists

def run_select(query):
    result = []

    try:
        db = mysql.connect(host=BD_HOST,
                database=BD_DATABASE,
                user=BD_USER,
                password=BD_PASS)
        cursor = db.cursor()
        cursor.execute(query)
        response = cursor.fetchall()
        field_names = [i[0] for i in cursor.description]
        for elem in response:
            result.append({})
            for index, value in enumerate(elem):
                result[-1][field_names[index]] = value
    except Exception as e:
        print(f"run_select - ERROR - {e}")
    finally:
        db.close()
    return result

def run_update(query):
	try:
		db = mysql.connect(host=BD_HOST,
							database=BD_DATABASE,
							user=BD_USER,
							password=BD_PASS)

		cursor = db.cursor()
		cursor.execute(query)
		db.commit()
	except Exception as e:
		print("run_update - ERROR " + str(e))
	finally:
		db.close()

def get_pending_and_unblocked_steps():
    query1 = 'SELECT * FROM neuralplatform_step WHERE status = "pending";'
    pending_steps = run_select(query1)
    unblocked_steps = []
    if len(pending_steps) > 0:
        pending_stepDefinitions = list(set([x['stepDefinition_id'] for x in pending_steps]))
        unblocked_stepDefinitions = []
        query2 = 'SELECT id, blockingStep_id FROM neuralplatform_stepdefinition;'
        stepDefinitions = run_select(query2)
        dependencies = {s['id']: s['blockingStep_id'] for s in stepDefinitions}
        for pending_stepDefinition in pending_stepDefinitions:
            sd_dependency = dependencies[pending_stepDefinition] or -1
            if not run_exists(f'SELECT id FROM neuralplatform_step WHERE status != "done" AND stepDefinition_id = {sd_dependency}'):
                unblocked_stepDefinitions.append(pending_stepDefinition)
        unblocked_steps = list(filter(lambda x: x['stepDefinition_id'] in unblocked_stepDefinitions, pending_steps))
    return unblocked_steps

def get_n_objects_by_key(objectName, n=1, key='id', keyValue=1):

    operator = '='
    if isinstance(keyValue, list):
        operator = 'IN'
        keyValue = '(' + str(keyValue)[1:-1] + ')'
    elif isinstance(keyValue, str):
        keyValue = '"' + keyValue + '"'
    query = f'SELECT * FROM neuralplatform_{objectName.lower()} WHERE {key} {operator} {keyValue};'
    elements = run_select(query)
    if len(elements) > 0:
        return elements[:n]
    return None

def update_object_by_key(objectName, key='id', keyValue=1, fields=None):

    if fields:
        operator = '='
        if isinstance(keyValue, list):
            operator = 'IN'
            keyValue = '(' + str(keyValue)[1:-1] + ')'
        elif isinstance(keyValue, str):
            keyValue = '"' + keyValue + '"'
        changes = ""
        for field in fields:
            value = fields[field]
            if isinstance(value, str):
                value = '"' + value + '"'
            changes += field + ' = ' + str(value) + ', '
        else:
            changes = changes[:-2]
        query = f'UPDATE neuralplatform_{objectName.lower()} SET {changes} WHERE {key} {operator} {keyValue};'
        run_update(query)

if __name__ == '__main__':
    update_object_by_key('StepDefinition', 'stepName', ['pdf2image', 'ocr', 'inference'], {'priority': 1})



import boto3
from .automl import AWS_ACC_KEY_ID, AWS_SEC_ACC_KEY, AWS_REGION_NAME

client_cb = boto3.client('codebuild',
						aws_access_key_id=AWS_ACC_KEY_ID,
						aws_secret_access_key=AWS_SEC_ACC_KEY,
						region_name=AWS_REGION_NAME)

FLASK_PROJECT 	= 'ALL_FLASK_Image_Builder'

def ecr_repository_from_codebuildProjectName(codebuildProjectName):
	project_name = codebuildProjectName.lower()
	for repo_name in ['ocr', 'predict', 'preprocess']:
		if repo_name in project_name:
			return f'flask_images_{repo_name}'
	return 'flask_images'

def build_image(codebuildProjectName, username, project_id):
	print(f"build_image : INFO : Building image for user: {username} and project: {project_id}...")
	image_tag = f'project_{project_id}'
	ecr_repository = ecr_repository_from_codebuildProjectName(codebuildProjectName)
	response = client_cb.start_build(
	    projectName=codebuildProjectName,
	    environmentVariablesOverride=[
	        {
	            'name': 'IMAGE_TAG',
	            'value': image_tag,
	            'type': 'PLAINTEXT'
	        },
			{
	            'name': 'USERNAME',
	            'value': username,
	            'type': 'PLAINTEXT'
	        },
			{
	            'name': 'PROJECT_ID',
	            'value': str(project_id),
	            'type': 'PLAINTEXT'
	        },
	    ]
	)
	print(f'build_image : INFO : Done!')
	if int(response['ResponseMetadata']['HTTPStatusCode']) == 200:
		return f'749868801319.dkr.ecr.us-west-2.amazonaws.com/{ecr_repository}:{image_tag}'
	else:
		return False



import boto3
from .automl import AWS_ACC_KEY_ID, AWS_SEC_ACC_KEY, AWS_REGION_NAME

client_batch = boto3.client('batch',
						aws_access_key_id=AWS_ACC_KEY_ID,
						aws_secret_access_key=AWS_SEC_ACC_KEY,
						region_name=AWS_REGION_NAME)

def create_job_definition():
	# REQUIERED PARAMS: jobDefinitionName, type

	'''
	EXAMPLE:

	{
		"jobDefinitionName": "first-run-job-definition",
		"jobDefinitionArn": "arn:aws:batch:us-west-2:749868801319:job-definition/first-run-job-definition:1",
		"revision": 1,
		"status": "ACTIVE",
		"type": "container",
		"parameters": {},
		"containerProperties": {
			"image": "749868801319.dkr.ecr.us-west-2.amazonaws.com/sagemaker-tf-cifar10-example:test_print",
			"vcpus": 1,
			"memory": 2000,
			"command": [],
			"jobRoleArn": "arn:aws:iam::749868801319:role/BasicTrainTaskRole",
			"volumes": [],
			"environment": [],
			"mountPoints": [],
			"ulimits": [],
			"resourceRequirements": []
		},
		"timeout": {
			"attemptDurationSeconds": 100
		}
	}
	'''
	response = client_batch.register_job_definition(
		jobDefinitionName='test-metrics',
		type='container',
		parameters={

		},
		containerProperties= {
			"image": "749868801319.dkr.ecr.us-west-2.amazonaws.com/sagemaker-tf-cifar10-example:test_metrics",
			"vcpus": 1,
			"memory": 2000,
			"command": [],
			"jobRoleArn": "arn:aws:iam::749868801319:role/BasicTrainTaskRole",
			"volumes": [],
			"environment": [],
			"mountPoints": [],
			"ulimits": [],
			"resourceRequirements": []
		},
		timeout={
			'attemptDurationSeconds': 60*60*5
		}
	)

def configure_instance(instance_type, environment):

	try:
		prior = 1 + ['SLOW','NORMAL','FAST'].index(instance_type.upper())
	except ValueError:
		return {'environment': environment}

	resources = []
	if prior > 1:
		resources = [
            {
                'value': f'{(prior-1)*2}', # NORMAL = 2	# FAST = 4
                'type': 'GPU'
            },
        ]

	containerOverrides={
        'vcpus': pow(4, prior),   # SLOW  = 4 		NORMAL = 16		FAST = 64
        'memory': 2048 * prior,   # SLOW  = 2GiB	NORMAL = 4GiB	FAST = 6GiB
        'environment': environment,
        'resourceRequirements': resources
    }
	return containerOverrides

def submit_train_job(params):
	print(f'submit_train_job : INFO : received params = {params}')
	environment = []

	for param in params:
		environment.append({'name': param, 'value': str(params[param])})

	containerOverrides = {'environment': environment}

	response = client_batch.submit_job(
		jobName=str(params['JOB_ID']),
		jobQueue='train-queue',
		jobDefinition=params['TRAINING_IMAGE'],
		parameters={},
		containerOverrides=containerOverrides,
		timeout={
        	'attemptDurationSeconds': params['TIMEOUT']
    	}
	)

	return response

def submit_job(params):
	print(f'submit_job : INFO : received params = {params}')
	environment = []

	for param in params:
		environment.append({'name': param, 'value': str(params[param])})

	containerOverrides = configure_instance(params['INSTANCE_TYPE'], environment)

	response = client_batch.submit_job(
		jobName=str(params['JOB_ID']),
		jobQueue='first-run-job-queue',
		jobDefinition=params['TRAINING_IMAGE'],
		parameters={},
		containerOverrides=containerOverrides,
		timeout={
        	'attemptDurationSeconds': 3600
    	}
	)

	return response

def submit_inf_req(params):
	print("Submit_inf_req - init")
	environment = []

	for param in params:
		environment.append({'name': param, 'value': str(params[param])})

	containerOverrides = configure_instance('Slow', environment)

	response = client_batch.submit_job(
		jobName=str(params['JOB_ID']),
		jobQueue='first-run-job-queue',
		jobDefinition='test-inference',
		parameters={},
		containerOverrides=containerOverrides,
		timeout={
        	'attemptDurationSeconds': 3600
    	}
	)

	return response

def custom_submit(cores):

	response = client_batch.submit_job(
		jobName='CONSOLE',
		jobQueue='first-run-job-queue',
		jobDefinition='test-inference',
		parameters={},
		containerOverrides={
	        'vcpus': cores,   # SLOW  = 4 		NORMAL = 16		FAST = 64
	        'memory': 512,   # SLOW  = 2GiB	NORMAL = 4GiB	FAST = 6GiB
	        'environment': [],
	        'resourceRequirements': []
	    },
		timeout={
        	'attemptDurationSeconds': 3600
    	}
	)

	return response

def launch_pretrain_job(params):

	print(f"launch_pretrain_job : INFO : launching pretrain job ({params['JOB_ID']}) for project ({params['PROJECT_ID']})...")
	environment = []

	for param in params:
		environment.append({'name': param, 'value': str(params[param])})

	# containerOverrides = configure_instance(params['INSTANCE_TYPE'], environment)

	response = client_batch.submit_job(
		jobName=str(params['JOB_ID']),
		jobQueue='pretrain-queue',
		jobDefinition=params['PRETRAIN_IMAGE'],
		parameters={},
		containerOverrides={'environment': environment},
		timeout={
        	'attemptDurationSeconds': 3600 * (1 + int(params['NPAGES'] / 1000))
    	}
	)
	print(f"launch_pretrain_job : INFO : submitted!")
	return response



from distutils.core import setup
setup(
  name = 'automlapi',
  packages = ['automlapi'],
  version = '0.96',
  license='MIT',
  description = 'api for the AutoML project',
  author = 'Raul Garcia',
  author_email = 'raulgfuentes97@gmail.com',
  url = 'https://github.com/GFuentesBSC/automlapi',
  download_url = 'https://github.com/GFuentesBSC/automlapi/archive/v0.96.tar.gz',
  keywords = ['AUTOML', 'BOTO3', 'AWS'],   # Keywords that define your package best
  install_requires=[
          'tqdm',
          'PyPDF2',
          'boto3',
          'botocore',
          'PyMySQL'
          # 'mysqlclient'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)



from .automl_batch import *
from .automl_cloudwatch import *
from .automl_cognito import *
from .automl_rds import *
from .automl_s3 import *
import getpass
import os
from tqdm import tqdm


class APIUser():

    def __init__(self):
        self.authenticated = False
        self.project_name = None
        self.login()

    def login(self):
        username = input("Username: ")
        password = getpass.getpass(prompt="Password: ")
        response = signin_user(username, password)
        if response['message'] == 'success':
            self.username = username
            self.password = password
            self.authenticated = True
            self.pk = get_user_pk_by_username_password(username, password)
            print("User and password correct")
        else:
            print("Wrong username or password")

    def upload_file(self, file_path):
        if self.authenticated:
            if None != self.project_name:
                file = open(file_path, "rb")
                response = upload_file_to_s3(file, self.username, self.project_name)
                paths = [ind_response[1] for ind_response in response if ind_response[0]]
                insert_files_to_rds(paths, self.project_name, self.pk)
                # for file_path in response:
                #     if ind_response[0]:
                #         if insert_file(str(ind_response[1]), self.project_name, self.pk):
                #             print("File uploaded: " + str(ind_response[1]))
                #         else:
                #             print("Database not updated for file: " + str(ind_response[1]))
                #     else:
                #         print("File could not be uploaded")
            else:
                # SELECT PROJECT first
                self.select_project()
        else:
            print("You are not logged-in, please log-in first")
            self.login()

    def select_project(self):
        if self.authenticated:
            project_names = get_projects_of_user(self.pk)
            if project_names == []:
                print("YOU DON'T HAVE PROJECTS!")
                return
            proj_name = ""
            while proj_name not in project_names:
                proj_name = input("Select one of your projects " + str(project_names) + ": ")
            self.project_name = proj_name
        else:
            print("You are not logged-in, please log-in first")
            self.login()



## PRIVATE FUNCTION TO UPLOAD DATASETS TO S3
def private_upload_dataset(folder, s3_folder):
    # s3_folder = 'data/CIFAR/train/'
    # s3_folder = 'data/CIFAR/validation/'
    os.chdir(folder)
    files = os.listdir()
    input("Gonna upload " + str(len(files)) + " files. Is that ok? ")
    for filename in tqdm(files):
        s3_path = os.path.join(s3_folder, filename)
        file = open(filename, "rb")
        if private_upload_file(file, s3_path) == False:
            print("path: " + s3_path)
            print("ERROR")
            break

## PRIVATE FUNCTION TO GENERATE 2 .LST FILES FROM A DIRECTORY (TRAIN AND TEST)
def make_lst(folder, ratio, extension):

    def find_label_in_name(filename, label_dict):
        for label in label_dict:
            if label in filename:
                return label_dict[label]
        return -1

    label_list = open(os.path.join(folder,'index.txt'), 'r').read().split(',')
    index = 0
    label_dict = dict()
    for l in label_list:
        label_dict[l] = index
        index += 1
    print("labels: " + str(label_dict))
    # filtrar los archivos de la carpeta por su extension (ex: si extension=='png' solo nos interesan los pngs)
    files = [x for x in os.listdir(folder) if x.endswith(extension.lower())]
    split_point = int(len(files)*ratio)
    train_files = files[:split_point]
    test_files = files[split_point:]

    train_lst_name = os.path.join(folder,'train.lst')
    test_lst_name = os.path.join(folder,'test.lst')
    errors_lst_name = os.path.join(folder,'errors.lst')
    f_train = open(train_lst_name, 'w')
    f_test = open(test_lst_name, 'w')
    f_errors = open(errors_lst_name, 'w')
    index = 0
    for filename in tqdm(train_files):
        label = find_label_in_name(filename, label_dict)
        row = str(index) + '\t' + str(label) + '\t' + filename
        if index < len(train_files) - 1:
            row += '\n'
        if label >= 0:
            f_train.write(row)
            index += 1
        else:
            f_errors.write(row)
    index = 0
    for filename in tqdm(test_files):
        label = find_label_in_name(filename, label_dict)
        row = str(index) + '\t' + str(label) + '\t' + filename
        if index < len(test_files) - 1:
            row += '\n'
        if label >= 0:
            f_test.write(row)
            index += 1
        else:
            f_errors.write(row)

    f_train.close()
    f_test.close()
    f_errors.close()



import boto3
from .automl import AWS_ACC_KEY_ID, AWS_SEC_ACC_KEY, AWS_REGION_NAME

client_ec2 = boto3.client('ec2',
						aws_access_key_id=AWS_ACC_KEY_ID,
						aws_secret_access_key=AWS_SEC_ACC_KEY,
						region_name=AWS_REGION_NAME)

def launch_instances_for_flask(num_instances, cluster_name, template_id, version):
	num_instances = min(num_instances, 20)
	num_instances = max(num_instances, 0)

	# indicate cluster
	userData = f'#!/bin/bash\necho ECS_CLUSTER={cluster_name} >> /etc/ecs/ecs.config\nyum update -y ecs-init\nsystemctl restart docker'

	response = client_ec2.run_instances(
		LaunchTemplate={
			'LaunchTemplateId': template_id,
			'Version': str(version)
		},
		MaxCount=num_instances,
		MinCount=num_instances,
		UserData=userData,
	)

	launched_instances_ids = [x['InstanceId'] for x in response['Instances']]
	return launched_instances_ids

def terminate_instances(instance_ids):
    response2 = client_ec2.terminate_instances(InstanceIds = instances_ids)
